package mirrgmods.plugincrops.framework;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.ListIterator;

import mirrgmods.plugincrops.deobf.HelperSrg;

import org.objectweb.asm.ClassReader;
import org.objectweb.asm.tree.AbstractInsnNode;
import org.objectweb.asm.tree.ClassNode;
import org.objectweb.asm.tree.FieldInsnNode;
import org.objectweb.asm.tree.InsnNode;
import org.objectweb.asm.tree.IntInsnNode;
import org.objectweb.asm.tree.JumpInsnNode;
import org.objectweb.asm.tree.LabelNode;
import org.objectweb.asm.tree.LdcInsnNode;
import org.objectweb.asm.tree.LineNumberNode;
import org.objectweb.asm.tree.MethodInsnNode;
import org.objectweb.asm.tree.MethodNode;
import org.objectweb.asm.tree.VarInsnNode;

import cpw.mods.fml.common.FMLLog;

public class HelperDeobf
{

	public static class HelperException extends Exception
	{

		public HelperException(String message)
		{
			super(message);
		}

		public HelperException(String message, Throwable cause)
		{
			super(message, cause);
		}

		public HelperException(Throwable cause)
		{
			super(cause);
		}

	}

	public static byte[] getBytesFromClassName(String className) throws HelperException
	{
		return getBytes("/" + className.replaceAll("\\.", "/") + ".class");
	}

	public static byte[] getBytes(String absolutePath) throws HelperException
	{
		URL url = HelperSrg.getResource(absolutePath);

		if (url == null) {
			throw new HelperException(absolutePath);
		}

		return getBytes(url);
	}

	public static byte[] getBytes(URL url) throws HelperException
	{
		InputStream is;
		try {
			is = url.openStream();
		} catch (IOException e) {
			throw new HelperException(url.toString(), e);
		}

		byte[] bytes;
		try {
			bytes = getBytes(is);
		} catch (HelperException e) {
			throw new HelperException(url.toString(), e);
		}

		return bytes;
	}

	public static byte[] getBytes(InputStream is) throws HelperException
	{
		ArrayList<byte[]> bytesList = new ArrayList<byte[]>();
		ArrayList<Integer> lengthList = new ArrayList<Integer>();

		while (true) {
			byte[] buf = new byte[2048];

			int res;
			try {
				res = is.read(buf);
			} catch (IOException e) {
				try {
					is.close();
				} catch (IOException e1) {
					throw new HelperException(e1);
				}
				throw new HelperException(e);
			}

			if (res == -1) {
				break;
			}

			bytesList.add(buf);
			lengthList.add(res);
		}

		int size = 0;
		for (int i = 0; i < lengthList.size(); i++) {
			size += lengthList.get(i);
		}

		byte[] buf = new byte[size];

		int index = 0;
		for (int i = 0; i < bytesList.size(); i++) {
			System.arraycopy(bytesList.get(i), 0, buf, index, lengthList.get(i));
			index += lengthList.get(i);
		}

		return buf;
	}

	public static MethodNode getMethodNode(ClassNode cnode, String name, String desc) throws NoSuchMethodException
	{
		MethodNode mnode = null;

		for (MethodNode curMnode : cnode.methods) {
			if (name.equals(curMnode.name) && desc.equals(curMnode.desc)) {
				mnode = curMnode;
				break;
			}
		}

		if (mnode == null) throw new NoSuchMethodException(name + desc);
		return mnode;
	}

	public static MethodNode getMethodNode(byte[] bytes, String name, String desc) throws NoSuchMethodException
	{
		ClassNode cnode = new ClassNode();
		ClassReader reader = new ClassReader(bytes);
		reader.accept(cnode, 0);

		return getMethodNode(cnode, name, desc);
	}

	public static void print(MethodNode mnode)
	{
		ListIterator<AbstractInsnNode> iterator = mnode.instructions.iterator();
		while (iterator.hasNext()) {
			AbstractInsnNode node = iterator.next();

			String str = "";

			str += node.getClass().getSimpleName();
			str += "[";
			str += node.getOpcode();
			str += "] ";

			if (node instanceof VarInsnNode) {
				str += String.format("%d", ((VarInsnNode) node).var);
			} else if (node instanceof JumpInsnNode) {
				str += String.format("%s", ((JumpInsnNode) node).label.getLabel());
			} else if (node instanceof LabelNode) {
				str += String.format("%s", ((LabelNode) node).getLabel());
			} else if (node instanceof MethodInsnNode) {
				str += String.format("%s %s %s", ((MethodInsnNode) node).owner, ((MethodInsnNode) node).name, ((MethodInsnNode) node).desc);
			} else if (node instanceof InsnNode) {
				str += String.format("%d", ((InsnNode) node).getType());
			} else if (node instanceof IntInsnNode) {
				str += String.format("%d", ((IntInsnNode) node).operand);
			} else if (node instanceof FieldInsnNode) {
				str += String.format("%s %s %s", ((FieldInsnNode) node).owner, ((FieldInsnNode) node).name, ((FieldInsnNode) node).desc);
			} else if (node instanceof LdcInsnNode) {
				str += String.format("%s %s", ((LdcInsnNode) node).cst.getClass(), ((LdcInsnNode) node).cst);
			} else if (node instanceof LineNumberNode) {
				str += String.format("<<<<< %d >>>>>", ((LineNumberNode) node).line);
			} else {
				str += String.format("%s %d", node.toString(), node.getType());
			}

			System.out.println(str);
		}
	}

	public static String dummyClassName = "mirrgmods.plugincrops.framework.DummyTileEntity";
	private static Boolean isDeobfuscated = null;

	public static boolean isDeobfuscated()
	{
		if (isDeobfuscated == null) {

			byte[] bytes;
			try {
				bytes = getBytesFromClassName(dummyClassName);
			} catch (HelperException e) {
				FMLLog.warning("[PluginCrops] helper error: file not found: '" + e + "'");
				if (e.getCause() != null) FMLLog.warning("[PluginCrops] " + e.getCause());
				isDeobfuscated = false;
				return isDeobfuscated;
			}

			MethodNode mnode;
			try {
				mnode = getMethodNode(bytes, "markDirty", "()V");
			} catch (NoSuchMethodException e) {
				isDeobfuscated = false;
				return isDeobfuscated;
			}

			isDeobfuscated = true;
			return isDeobfuscated;
		}

		return isDeobfuscated;
	}

}
